class Commands:
    LEECH = "leech"
    PURGE = "purge"
    PAUSEALL = "pauseall"
    RESUMEALL = "resumeall"
    STATUS = "status"
    SETTINGS = "settings"
    EXEC = "exec"
    UPLOAD = "upload"
    YTDL = "ytdl"
    PYTDL = "pytdl"
    ABOUT = "about"
    GETLOGS = "getlogs"
    SERVER = "stats"
    USERSETTINGS = "usettings"
    INSTADL = "instadl"
    START = "start"
    USTATUS = "ustatus"
    SETTHUMB = "setthumb"
    CLRTHUMB = "clearthumb"
    SPEEDTEST = "speedtest"